-module(gmc_grids).
-vsn("0.1.0").
-author("Craig Everett <craigeverett@qpq.swiss>").
-copyright("QPQ AG <info@qpq.swiss>").
-license("GPL-3.0-or-later").

-export([parse/1]).
-include("$zx_include/zx_logger.hrl").


-spec parse(URL) -> {ok, Instruction} | {error, Reason}
    when URL         :: string(),
         Instruction :: {{spend, chain | node}, {Location, Recipient, Amount, Payload}}
                      | {{sign, http | https}, URL},
         Location    :: Node  :: {inet:ip_address() | inet:hostname(), inet:port_number()}
                      | Chain :: binary(),
         Recipient   :: clutch:id(),
         Amount      :: non_neg_integer(),
         Payload     :: binary(),
         URL         :: string(),
         Reason      :: bad_url.

parse(URL) ->
    case uri_string:parse(URL) of
        #{path := "/1/s/" ++ R, host := H, query := Q, scheme := "grids"} ->
            spend(R, chain, list_to_binary(H), Q);
        #{path := "/1/s/" ++ R, host := H, query := Q, scheme := "grid"} ->
            spend(R, chain, list_to_binary(H), Q);
        #{path := "/1/t/" ++ R, host := H, port := P, query := Q, scheme := "grids"} ->
            spend(R, node, {H, P}, Q);
        #{path := "/1/t/" ++ R, host := H, port := P, query := Q, scheme := "grid"} ->
            spend(R, node, {H, P}, Q);
        #{path := "/1/t/" ++ R, host := H, query := Q, scheme := "grids"} ->
            spend(R, node, {H, 3013}, Q);
        #{path := "/1/t/" ++ R, host := H, query := Q, scheme := "grid"} ->
            spend(R, node, {H, 3013}, Q);
        U = #{path := "/1/d/" ++ L, scheme := "grids"} ->
            NewURL = uri_string:recompose(U#{scheme := "https", path := L}),
            {ok ,{{sign, https}, NewURL}};
        U = #{path := "/1/d/" ++ L, scheme := "grid"} ->
            NewURL = uri_string:recompose(U#{scheme := "http", path := L}),
            {ok, {{sign, http}, NewURL}};
        {error, Reason, Info} ->
            ok = tell("URL parsing failed with ~w: ~p", [Reason, Info]),
           {error, bad_url};
        U ->
            ok = tell("GRIDS cannot proceed with this result: ~p", [U]),
            {error, bad_url}
    end.

spend(Recipient, Context, Location, Qwargs) ->
    case dissect_query(Qwargs) of
        {ok, Amount, Payload} ->
            
            {ok, {{spend, Context}, {Location, Recipient, Amount, Payload}}};
        Error ->
            Error
    end.


dissect_query(Qwargs) ->
    case uri_string:dissect_query(Qwargs) of
        {error, Reason, Info} ->
            ok = tell("URL parsing failed with ~w: ~p", [Reason, Info]),
            {error, bad_url};
        ArgList ->
            Amount  = proplists:get_value("a", ArgList, 0),
            Payload = proplists:get_value("p", ArgList, <<>>),
            {ok, Amount, Payload}
    end.
    
